@@ -2,7 +2,10 @@
 
 use core::fmt::{self, Display};
 
-/// Result type.
+#[cfg(feature = "pkcs8")]
+use crate::pkcs8;
+
+/// Result type with the `elliptic-curve` crate's [`Error`] type.
 pub type Result<T> = core::result::Result<T, Error>;
 
 /// Elliptic curve errors.
@@ -22,5 +25,12 @@ impl From<pkcs8::Error> for Error {
     }
 }
 
+#[cfg(feature = "sec1")]
+impl From<sec1::Error> for Error {
+    fn from(_: sec1::Error) -> Error {
+        Error
+    }
+}
+
 #[cfg(feature = "std")]
 impl std::error::Error for Error {}
